<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Classes\Finance\BankTransferHelper;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\BankTransfer;
use App\Models\Main\Bank;
use App\Models\Main\DailyType;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Customer;
use App\Models\Sales\Clearance;
use Illuminate\Http\Request;
use DB;

class BankTransferController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('02-017','view');


        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'status' => $request['status'] ?? 1 ,
            'rows'   => $request['rows'] ?? 20 ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-017', $lang)->sub_title ?? '';
        $this->return['items']       = BankTransfer::all_bank_transfers($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('02-017','add');

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-017', $lang)->sub_title ?? '';
        $this->return['banks']       = Bank::all_banks(array('active' => 1, 'lang' => $lang));
        $this->return['customers']   = Customer::all_customers(array('active' => 1, 'lang' => $lang));
        $this->return['daily_types'] = DailyType::all_daily_types(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('02-017','add');


        $insert = $request->only(['action_date', 'daily_type_id', 'arabic_description', 'items']);

        $year   = array_search(date("Y", strtotime($insert['action_date'])), $this->data['years']);

        if($year == ''){
            return $this->sendError(trans("error.finance_year_not_aval"));
        }

        $codes = (new BankTransferHelper)->get_next_codes($insert['action_date'], $year, $this->data['current_branch']);

        $master_seed = [
            'branch_id'          => $this->data['current_branch'],
            'finance_year_id'    => $year,
            'year_ser'           => $codes['year_ser'],
            'month_ser'          => $codes['month_ser'],
            'branch_ser'         => $codes['branch_ser'],
            'daily_journal_m_id' => 0,
            'action_date'        => $insert['action_date'],
            'daily_type_id'      => $insert['daily_type_id'],
            'arabic_description' => $insert['arabic_description'],
            'total_amount'       => collect($insert['items'])->sum('amount'),
            'add_user'           => auth()->user()->id
        ];

        $validator = validator()->make($master_seed, BankTransfer::$master_rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        DB::beginTransaction();

        $m_id = BankTransfer::add_bank_transfer_m($master_seed);

        if (isset($insert['items']) && is_array($insert['items'])){
            foreach ($insert['items'] as $item){
                $details_seed = array(
                    'bank_transfer_m_id' => $m_id,
                    'transfer_from'      => $item['transfer_from'] ?? 0,
                    'from_bank_id'       => $item['from_bank_id'] ?? 0,
                    'from_customer_id'   => $item['from_customer_id'] ?? 0,
                    'transfer_to'        => $item['transfer_to'] ?? 0,
                    'to_bank_id'         => $item['to_bank_id'] ?? 0,
                    'to_customer_id'     => $item['to_customer_id'] ?? 0,
                    'clearance_id'       => $item['clearance_id'] ?? 0,
                    'arabic_description' => $item['arabic_description'] ?? 0,
                    'amount'             => $item['amount'] ?? 0,
                    'add_user'           => auth()->user()->id
                );

                $validator = validator()->make($details_seed, BankTransfer::$details_rules);

                if($validator->fails())
                {
                    return $this->sendError(trans('error.add_fails'), $validator->errors());
                }

                $d_id = BankTransfer::add_bank_transfer_d($details_seed);
            }
        }

        $dailyJournal = (new BankTransferHelper)->generate_journal($m_id);

        if(!$dailyJournal[0]){
            DB::rollback();
            return response()->json(array('check' => 0 , 'data' => $dailyJournal[3] ?? [], 'msg' => 'Error In Creating Journal Code '.$dailyJournal[1]));
        }

        DB::table('bank_transfer_m')
            ->where('bank_transfer_m_id' , $m_id)
            ->update(['daily_journal_m_id' => $dailyJournal[2]]);

        DB::commit();
        return $this->sendResponse([], trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('02-017','view');

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-017' , $lang)->sub_title ?? '';
        $this->return['master']      = BankTransfer::get_bank_transfer_m($id, $lang);
        $this->return['details']     = BankTransfer::get_bank_transfer_d($id, $lang);

        $this->return['banks']       = Bank::all_banks(array('active' => 1, 'lang' => $lang));
        $this->return['customers']   = Customer::all_customers(array('active' => 1, 'lang' => $lang));
        $this->return['daily_types'] = DailyType::all_daily_types(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('02-017','edit');


        $update = $request->only(['action_date', 'daily_type_id', 'arabic_description']);

        BankTransfer::update_bank_transfer_m($update, $id);
        return $this->sendResponse($id, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-017','delete');


        $master = BankTransfer::get_bank_transfer_m($id, $lang);

        if(!isset($master->bank_transfer_m_id) || $master->branch_id != $this->data['current_branch'] || !(new BankTransferHelper)->can_edit_or_delete($id)){
            return $this->sendError(trans("error.delete_fail"));
        }

        BankTransfer::delete_bank_transfer_m($id);

        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = BankTransfer::find($id);

        if($item->bank_transfer_active){
            $this->checkPermission('02-017','delete');


            if(!(new BankTransferHelper)->can_edit_or_delete($id)){
                return $this->sendError( trans('error.delete_fails'), []);
            }

            BankTransfer::delete_bank_transfer_m($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('02-017','edit');

            if(!(new BankTransferHelper)->can_edit_or_delete($id)){
                return $this->sendError( trans('error.update_fails'), []);
            }

            BankTransfer::update_bank_transfer_m(array('bank_transfer_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
